/*
 *  LibRoadnav
 *  Direction.h
 *
 *  Copyright (c) 2004 - 2007 Richard L. Lynch <rllynch@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef INCLUDED_DIRECTION_H_
#define INCLUDED_DIRECTION_H_

#include <wx/string.h>

class Angle;

//////////////////////////////////////////////////////////////////////////////
///
/// \brief Represents a direction
///
//////////////////////////////////////////////////////////////////////////////
class Direction
{
public:
   // GROUP: LIFECYCLE

   //------------------------------------------------------------------------
   // Default constructor
   Direction();

   //------------------------------------------------------------------------
   // Direction defined as bearing
   Direction(char n,unsigned short deg,unsigned short min,double sec,char e);

   //------------------------------------------------------------------------
   // Direction defined as azimuth
   Direction(short deg,unsigned short min,double sec);

   //------------------------------------------------------------------------
   // Direction defined as azimuth
   Direction(const Angle& rAzimuth);

   //------------------------------------------------------------------------
   Direction(double direction);

   //------------------------------------------------------------------------
   // Copy constructor
   Direction(const Direction& rOther);

   //------------------------------------------------------------------------
   // Destructor
   ~Direction();

   // GROUP: OPERATORS

   //------------------------------------------------------------------------
   // Assignment operator
   //
   // Returns reference to itself
   Direction& operator = (const Direction& rOther);

   //------------------------------------------------------------------------
   Direction& operator = (double direction);

   //------------------------------------------------------------------------
   bool operator == (const Direction& rOther) const;

   //------------------------------------------------------------------------
   bool operator != (const Direction& rOther) const;

   //------------------------------------------------------------------------
   bool operator < (const Direction& rOther) const;

   //------------------------------------------------------------------------
   Direction& operator+=(const Angle& angle);

   //------------------------------------------------------------------------
   Direction& operator-=(const Angle& angle);

   // GROUP: OPERATIONS

   //------------------------------------------------------------------------
   Direction& Increment(const Angle& angle);

   //------------------------------------------------------------------------
   Direction IncrementBy(const Angle& angle) const;

   //------------------------------------------------------------------------
   // Converts a bearing to an angle measured clockwise from due north in
   // radians.
   static double BearingToDBearing(char n,unsigned short deg,unsigned short min,double sec,char e);

   //------------------------------------------------------------------------
   // Converts an angle measured clockwise from due north in radians to a
   // bearing.
   static void DBearingToBearing(double dir,char& n,unsigned short& deg,unsigned short& min,double& sec,char& e);

   //------------------------------------------------------------------------
   static bool BearingFromString(const char* str,Direction* pDir);

   //------------------------------------------------------------------------
   // Converts an azimuth to an angle measured clockwise from due north in
   // radians
   static double AzimuthToDAzimuth(short deg,unsigned short min,double sec);

   //------------------------------------------------------------------------
   // Converts an angle measued clockwise from due north in radians to an
   // azimuth in deg-min-sec format.
   static void DAzimuthToAzimuth(double angel,short& deg,unsigned short& min,double& sec);

   // GROUP: ACCESS

   //------------------------------------------------------------------------
   // Returns the direction as a floating point value.  The direction is an
   // angle measured clockwise from due north, in radians.
   double AsDouble() const;

   //------------------------------------------------------------------------
   wxString AsBearingString(bool bFull = true,bool bSymbols = false) const;

   //------------------------------------------------------------------------
   wxString AsAzimuthString(bool bSymbols = false) const;

   // GROUP: INQUIRY

protected:
   // GROUP: DATA MEMBERS
   // GROUP: LIFECYCLE
   // GROUP: OPERATORS
   // GROUP: OPERATIONS
   void MakeCopy(const Direction& rOther);
   void MakeAssignment(const Direction& rOther);

   // GROUP: ACCESS
   // GROUP: INQUIRY

private:
   // GROUP: DATA MEMBERS
   double m_Direction; // Direction of line. Measured zero radians due north 
                       // and increasing in a CW direction.

   // GROUP: LIFECYCLE
   // GROUP: OPERATORS
   // GROUP: OPERATIONS
   void Normalize();

   //------------------------------------------------------------------------
   void AsDmsString(wxString& str,bool bSymbols = false) const;
};

// INLINE METHODS
//

// EXTERNAL REFERENCES
//

//------------------------------------------------------------------------
const Direction operator+(const Direction& dir,const Angle& angle);

//------------------------------------------------------------------------
const Direction operator-(const Direction& dir,const Angle& angle);

//------------------------------------------------------------------------
const Angle operator-(const Direction& dir1,const Direction& dir2);


#endif // INCLUDED_DIRECTION_H_
